library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.Numeric_Std.all;
library UNISIM;
use UNISIM.vcomponents.all;

entity AD936x_IOInterface is
	port (	---------------------
				-- clock and reset --
				---------------------	
				Clock						:  in std_logic;								
				Reset_ALow				:  in std_logic;								-- Reset, active low
				---------------------------------
				-- Interface to FMC PinWrapper --
				---------------------------------
				AD_Enable				: out std_logic;								
				AD_ResetB				: out std_logic;								
				AD_Data_Clk				:  in std_logic;  									
				AD_FB_Clk				: out std_logic;								
				AD_Data_Rx				:  in std_logic_vector(11 downto 0);		
				AD_Data_Tx				: out std_logic_vector(11 downto 0);		
				AD_Rx_Frame				:  in std_logic;								
				AD_Tx_Frame				: out std_logic;								
				AD_TxNRx					: out std_logic;								
				AD_En_AGC				: out std_logic;								
				AD_Ctrl_Out				:  in std_logic_vector(7 downto 0);		
				AD_Ctrl_In				: out std_logic_vector(3 downto 0);   	
				AD_Sync_In				: out std_logic;								
				AD_ClkOut				:  in std_logic;								
				----------------------------------------------
				-- AXI4 Stream Connection to Data Interface --
				----------------------------------------------
				AD_Rx_Valid				: out std_logic;								
				AD_Rx_Data				: out std_logic_vector(31 downto 0);
				AD_Rx_Ack				:  in std_logic;					
				AD_Tx_Valid				:  in std_logic;								
				AD_Tx_Data				:  in std_logic_vector(31 downto 0);	
				AD_Tx_Ack				: out std_logic;
				----------------------------------------------------
				-- AXI4 Stream Connection to Parameter Processing --
				----------------------------------------------------
				Samples_FIFO_Enable	:  in std_logic;	
				Samples_Clk				:  in std_logic;			
				Samples_Rx_Valid		: out std_logic;									
				Samples_Rx_Data		: out std_logic_vector(31 downto 0);		
				Samples_Rx_Ack			:  in std_logic;									
				Samples_Tx_Valid		: out std_logic;									
				Samples_Tx_Data		: out std_logic_vector(31 downto 0);		
				Samples_Tx_Ack			:  in std_logic;					
				---------------------
				-- control signals --
				---------------------
				AD936x_Activated		:  in std_logic);								-- (de)activate AD936x transceiver
end AD936x_IOInterface;

architecture Behavioral of AD936x_IOInterface is
signal 	AD_Rx_Valid_Internal, AD_Tx_Ack_Internal, AD_Tx_Valid_Internal, AD_Tx_Frame_Internal	: std_logic;
signal 	AD_Rx_Data_Internal, AD_Tx_Data_Internal, AD_Tx_Data_IQ											: std_logic_vector(31 downto 0);
begin


	--------------------------------------------------------------------
	-- current configuration														--
	-- Dual Port Full Duplex Mode(CMOS), Double Data Rate (DDR), 1R1T --
	--------------------------------------------------------------------

	-------------------------------
	-- control signals to AD936x --
	-------------------------------
	
	AD_Enable 	<= '1';
	AD_ResetB 	<= AD936x_Activated;														
	AD_TxNRx 	<= '1';																		
	AD_Sync_In	<= '0';																		
	AD_En_AGC	<= '1';		
	AD_Ctrl_In	<= "0001";


	--------------------------
	-- transmit AD936x data --
	--------------------------


	-- set FB clock
	FB_Clk_ODDR2: ODDR2 
	generic map	(	DDR_ALIGNMENT	=> "NONE",										-- Sets output alignment to "NONE", "C0", "C1"
						INIT				=> '0',											-- Sets initial state of the Q output to '0' or '1
						SRTYPE			=> "SYNC")										-- Specifies "SYNC" or "ASYNC" set/reset
   port map		(	Q	=> AD_FB_Clk,  												-- 1-bit DDR output data
						C0 => AD_Data_Clk, 												-- 1-bit clock input
						C1	=> not AD_Data_Clk, 											-- 1-bit clock input
						CE	=> '1', 															-- 1-bit clock enable input
						D0	=> '1', 															-- 1-bit data input (associated with C0)
						D1 => '0', 															-- 1-bit data input (associated with C1)
						R	=> not Reset_ALow,  											-- 1-bit reset input, active high
						S	=> '0');															-- 1-bit set input


	-- interaction with data interface fifo
	Set_IQ: process(AD_Data_Clk, Reset_ALow)
	begin
		if Reset_ALow='0' then 	AD_Tx_Data_IQ 			<= (others => '0');
										AD_Tx_Frame_Internal	<= '0';
		
		elsif falling_edge(AD_Data_Clk) then
			-- default
			AD_Tx_Data_IQ 			<= (others => '0');
			AD_Tx_Frame_Internal	<= '0';
			-- check for new fifo data
			if AD_Tx_Valid_Internal='1' then
				-- save IQ word
				AD_Tx_Data_IQ <= AD_Tx_Data_Internal;
				-- set Tx frame value
				AD_Tx_Frame_Internal	<= '1';
			end if;
		end if;
	end process Set_IQ;
	
	-- ack fifo asynchronously
	AD_Tx_Ack_Internal <= '1' when AD_Tx_Valid_Internal='1' else '0';


	-- set Tx Frame
	Tx_Frame_ODDR2: ODDR2 
	generic map	(	DDR_ALIGNMENT	=> "NONE",										-- Sets output alignment to "NONE", "C0", "C1"
						INIT				=> '0',											-- Sets initial state of the Q output to '0' or '1
						SRTYPE			=> "SYNC")										-- Specifies "SYNC" or "ASYNC" set/reset
   port map		(	Q	=> AD_Tx_Frame,  												-- 1-bit DDR output data
						C0 => AD_Data_Clk, 												-- 1-bit clock input
						C1	=> not AD_Data_Clk, 											-- 1-bit clock input
						CE	=> '1', 															-- 1-bit clock enable input
						D0	=> AD_Tx_Frame_Internal, 									-- 1-bit data input (associated with C0)
						D1 => '0', 															-- 1-bit data input (associated with C1)
						R	=> not Reset_ALow,  											-- 1-bit reset input, active high
						S	=> '0');															-- 1-bit set input
						
						
	-- generate double data rate IQ output
	Gen_DDR_Output: for I in 0 to 11 generate
	
		Data_P1_ODDR2: ODDR2 
		generic map	(	DDR_ALIGNMENT	=> "NONE",									-- Sets output alignment to "NONE", "C0", "C1"
							INIT				=> '0',										-- Sets initial state of the Q output to '0' or '1
							SRTYPE			=> "SYNC")									-- Specifies "SYNC" or "ASYNC" set/reset
		port map		(	Q	=> AD_Data_Tx(I),  										-- 1-bit DDR output data
							C0 => AD_Data_Clk, 											-- 1-bit clock input
							C1	=> not AD_Data_Clk, 										-- 1-bit clock input
							CE	=> '1', 														-- 1-bit clock enable input
							D0	=> AD_Tx_Data_IQ(I+16),	-- I							-- 1-bit data input (associated with C0)
							D1 => AD_Tx_Data_IQ(I),		-- Q							-- 1-bit data input (associated with C1)
							R	=> not Reset_ALow,  										-- 1-bit reset input, active high
							S	=> '0');														-- 1-bit set input

	end generate Gen_DDR_Output;

	
	-------------------------
	-- receive AD936x data --
	-------------------------


	-- generate the valid signal from the rx frame signal
	Get_Rx_Frame: process(AD_Data_Clk, Reset_ALow)
	begin
		if Reset_ALow='0' then AD_Rx_Valid_Internal <= '0';
		
		elsif falling_edge(AD_Data_Clk) then
			-- save signal
			AD_Rx_Valid_Internal <= AD_Rx_Frame;
		end if;
	end process Get_Rx_Frame;
	
		
	-- receive double data rate IQ data 
	Get_DDR_Input: for I in 0 to 11 generate
	
		Data_P0_IDDR2: IDDR2
		generic map	(	DDR_ALIGNMENT 	=> "NONE", 										-- Sets output alignment to "NONE", "C0", "C1"
							INIT_Q0 			=> '0', 											-- Sets initial state of the Q0 output to '0' or '1'
							INIT_Q1 			=> '0', 											-- Sets initial state of the Q1 output to '0' or '1'
							SRTYPE 			=> "SYNC") 										-- Specifies "SYNC" or "ASYNC" set/reset
		port map 	(	Q0 => AD_Rx_Data_Internal(I), 	-- Q						-- 1-bit output captured with C0 clock
							Q1 => AD_Rx_Data_Internal(16+I), -- I						-- 1-bit output captured with C1 clock
							C0 => AD_Data_Clk, 												-- 1-bit clock input
							C1 => not AD_Data_Clk, 											-- 1-bit clock input
							CE => '1', 															-- 1-bit clock enable input
							D 	=> AD_Data_Rx(I),												-- 1-bit data input
							R 	=> not Reset_ALow, 											-- 1-bit reset input, active high
							S 	=> '0');															-- 1-bit set input
		
	end generate Get_DDR_Input;


	-- resize the data vector to 16 Bit IQ, 2's complement
	AD_Rx_Data_Internal(31 downto 28) <= AD_Rx_Data_Internal(27) & AD_Rx_Data_Internal(27) & AD_Rx_Data_Internal(27) & AD_Rx_Data_Internal(27);
	AD_Rx_Data_Internal(15 downto 12) <= AD_Rx_Data_Internal(11) & AD_Rx_Data_Internal(11) & AD_Rx_Data_Internal(11) & AD_Rx_Data_Internal(11);
	

	--------------------
	-- IO data FIFOs  --
	--------------------
	
	
	-- direction Connector -> User FPGA
	AD_FIFO_Out: entity Work.AXI4_FIFO_16x32
	port map	(	-- system --
					s_aresetn 		=> Reset_ALow,
					-- write port = AD936x Interface --
					s_aclk 			=> not AD_Data_Clk,
					s_axis_tvalid 	=> AD_Rx_Valid_Internal,
					s_axis_tdata 	=> AD_Rx_Data_Internal,
					s_axis_tready 	=> open,												-- not neccessary, because we can't stop the rx data
					-- read port = core --
					m_aclk 			=> Clock, 
					m_axis_tvalid 	=> AD_Rx_Valid,						
					m_axis_tdata 	=> AD_Rx_Data,
					m_axis_tready 	=> AD_Rx_Ack);												
					

	-- direction User FPGA -> Connector
	AD_FIFO_In: entity Work.AXI4_FIFO_16x32
	port map	(	-- system --
					s_aresetn 		=> Reset_ALow,
					-- write port = core --
					s_aclk 			=> Clock,	 
					s_axis_tvalid 	=> AD_Tx_Valid,
					s_axis_tdata 	=> AD_Tx_Data,
					s_axis_tready 	=> AD_Tx_Ack,								
					-- read port = AD936x Interface --
					m_aclk 			=> not AD_Data_Clk, 
					m_axis_tvalid 	=> AD_Tx_Valid_Internal,														
					m_axis_tdata 	=> AD_Tx_Data_Internal,
					m_axis_tready 	=> AD_Tx_Ack_Internal);


	------------------------------------------------------------
	-- Data FIFOs for protocol packet snapshots (display FFT) --
	------------------------------------------------------------
	
	
	Rx_Samples_FIFO: entity Work.AXI4_FIFO_1024x32
	port map	(	-- general --
					s_aresetn 		=> Samples_FIFO_Enable,
					-- write port --
					s_aclk 			=> not AD_Data_Clk,
					s_axis_tvalid 	=> AD_Rx_Valid_Internal,
					s_axis_tdata 	=> AD_Rx_Data_Internal,
					s_axis_tready 	=> open,												
					-- read ports --
					m_aclk 			=> Samples_Clk, 
					m_axis_tvalid 	=> Samples_Rx_Valid,						
					m_axis_tdata 	=> Samples_Rx_Data,
					m_axis_tready 	=> Samples_Rx_Ack);


	Tx_Samples_FIFO: entity Work.AXI4_FIFO_1024x32
	port map	(	-- general --
					s_aresetn 		=> Samples_FIFO_Enable,
					-- write port --
					s_aclk 			=> not AD_Data_Clk,
					s_axis_tvalid 	=> AD_Tx_Valid_Internal and AD_Tx_Ack_Internal,
					s_axis_tdata 	=> AD_Tx_Data_Internal,
					s_axis_tready 	=> open,												
					-- read ports --
					m_aclk 			=> Samples_Clk, 
					m_axis_tvalid 	=> Samples_Tx_Valid,						
					m_axis_tdata 	=> Samples_Tx_Data,
					m_axis_tready 	=> Samples_Tx_Ack);
					
					
end Behavioral;

