library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
library UNISIM;
use UNISIM.vcomponents.all;

entity DataProcessing is
	port (	---------------------
				-- clock and reset --
				---------------------
				Clock						:  in std_logic;									
				Reset_ALow				:  in std_logic;									-- Reset, active low
				----------------------------------------------
				-- AXI4 Stream Connection to AD936x Control --
				----------------------------------------------
				AD_Rx_Valid				:  in std_logic;									
				AD_Rx_Data				:  in std_logic_vector(31 downto 0);
				AD_Rx_Ack				: out std_logic;					
				AD_Tx_Valid				: out std_logic;									
				AD_Tx_Data				: out std_logic_vector(31 downto 0);		
				AD_Tx_Ack				:  in std_logic;									
				----------------------------------------------
				-- AXI4 Stream Data Connection to User FPGA --
				----------------------------------------------
				Data_Rx_Clk				:    in std_logic;								
				Data_Rx_Valid			:    in std_logic;								
				Data_Rx_Data			:    in std_logic_vector(15 downto 0);		
				Data_Rx_Ack				:   out std_logic;								
				Data_Tx_Clk				:   out std_logic;								
				Data_Tx_Valid			:   out std_logic;								
				Data_Tx_Data			:   out std_logic_vector(15 downto 0);		
				Data_Tx_Ack				:    in std_logic);																
end DataProcessing;

architecture Behavioral of DataProcessing is
signal AD_Rx_Full, Data_Rx_Valid_Internal, Data_Rx_Ack_Internal, Data_Tx_Valid_Internal,
		 Data_Tx_Ack_Internal, AD_Rx_Valid_Internal, AD_Rx_Ack_Internal								: std_logic;
signal Data_Rx_Data_Internal, Data_Tx_Data_Internal														: std_logic_vector(15 downto 0);
signal AD_Rx_Data_Internal, AD_Tx_Data_Internal																: std_logic_vector(31 downto 0);
begin


	------------------------------------------------------------------------------
	-- data input processing for Tx direction (User FPGA -> FMC Connector FPGA) --
	------------------------------------------------------------------------------

	-- clock in/out all signals
	User2FMCClkIn: process(Data_Rx_Clk, Reset_ALow)
	begin
		if Reset_ALow='0' then	Data_Rx_Valid_Internal	<= '0';
										Data_Rx_Data_Internal	<= (others => '0');
										Data_Rx_Ack 				<= '0';
		
		elsif rising_edge(Data_Rx_Clk) then
			Data_Rx_Valid_Internal	<= Data_Rx_Valid;
			Data_Rx_Data_Internal	<= Data_Rx_Data;
			Data_Rx_Ack 				<= not Data_Rx_Ack_Internal;
		end if;
	end process User2FMCClkIn;
	

	-- we use two small fifos for buffering the incoming data and convert to 32 Bit format --
	FIFO_User2FMC: entity Work.Native_FIFO_2048x16_1024x32
		port map (	-- general --
						rst 				=> not Reset_ALow,
						-- write ports --
						wr_clk 			=> Data_Rx_Clk,
						wr_en 			=> Data_Rx_Valid_Internal,
						din 				=> Data_Rx_Data_Internal,
						full 				=> open,
						prog_full		=> Data_Rx_Ack_Internal,					
						-- read ports --	 
						rd_clk 			=> Clock,						
						valid 			=> AD_Tx_Valid,										
						dout 				=> AD_Tx_Data,
						rd_en 			=> AD_Tx_Ack,
						empty 			=> open);		
	

	-------------------------------------------------------------------------------
	-- data output processing for Rx direction (FMC Connector FPGA -> User FPGA) --
	-------------------------------------------------------------------------------
	
	
	-- data buffer
	Data_FIFO_Out: entity Work.AXI4_FIFO_8192x32
	port map	(	-- Systemsignale --
					s_aresetn 		=> Reset_ALow,
					-- write ports --
					s_aclk 			=> Clock,	 
					s_axis_tvalid 	=> AD_Rx_Valid,
					s_axis_tdata 	=> AD_Rx_Data,
					s_axis_tready 	=> AD_Rx_Ack,												
					-- read ports --
					m_aclk 			=> Clock, 
					m_axis_tvalid 	=> AD_Rx_Valid_Internal,						
					m_axis_tdata 	=> AD_Rx_Data_Internal,
					m_axis_tready 	=> AD_Rx_Ack_Internal);


	-- invert full flag to get the ack information 
	AD_Rx_Ack_Internal <= not AD_Rx_Full;
	
	
	-- we use two small fifos for buffering the outgoing data and convert to 16 Bit format --
	FIFO_FMC2User: entity Work.Native_FIFO_1024x32_2048x16 
	port map (	-- general --
					rst 				=> not Reset_ALow,
					-- write ports --
					wr_clk 			=> Clock,
					wr_en 			=> AD_Rx_Valid_Internal,
					din 				=> AD_Rx_Data_Internal,
					full 				=> AD_Rx_Full,											
					-- read ports --	 
					rd_clk 			=> Clock,						
					valid 			=> Data_Tx_Valid_Internal,
					dout 				=> Data_Tx_Data_Internal,
					rd_en 			=> Data_Tx_Ack_Internal,
					empty 			=> open);


	-- clock in/out all singnals
	FMC2UserClkOut: process(Clock, Reset_ALow)
	begin
		if Reset_ALow='0' then	Data_Tx_Valid 			<= '0';
										Data_Tx_Data			<= (others => '0');
										Data_Tx_Ack_Internal	<= '0';
		
		elsif rising_edge(Clock) then
			Data_Tx_Valid 			<= Data_Tx_Valid_Internal and Data_Tx_Ack_Internal;
			Data_Tx_Data			<= Data_Tx_Data_Internal;
			Data_Tx_Ack_Internal	<= Data_Tx_Ack;
		end if;
	end process FMC2UserClkOut;


	-- provide tx clock --
	Clk_FMC2User: ODDR2 
	generic map	(	DDR_ALIGNMENT	=> "NONE",										-- Sets output alignment to "NONE", "C0", "C1"
						INIT				=> '0',											-- Sets initial state of the Q output to '0' or '1
						SRTYPE			=> "SYNC")										-- Specifies "SYNC" or "ASYNC" set/reset
   port map		(	Q	=> Data_Tx_Clk,  												-- 1-bit DDR output data
						C0 => Clock, 														-- 1-bit clock input
						C1	=> not Clock, 													-- 1-bit clock input
						CE	=> '1', 															-- 1-bit clock enable input
						D0	=> '1', 															-- 1-bit data input (associated with C0)
						D1 => '0', 															-- 1-bit data input (associated with C1)
						R	=> not Reset_ALow,  											-- 1-bit reset input, active high
						S	=> '0');															-- 1-bit set input


end Behavioral;

