library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity Protocol is
	port	(	---------------------
				-- clock and reset --
				---------------------				
				Clock						:  in std_logic;									
				Reset_ALow				:  in std_logic;									-- Reset,active low
				---------------------------------------------------
				-- AXI4 Stream Connection to Parameter Interface --
				---------------------------------------------------											
				Ctrl_Rx_Valid			:    in std_logic;								
				Ctrl_Rx_Data			:    in std_logic_vector(7 downto 0);		
				Ctrl_Rx_Ack				:   out std_logic;																
				Ctrl_Tx_Valid			:   out std_logic;								
				Ctrl_Tx_Data			:   out std_logic_vector(7 downto 0);		
				Ctrl_Tx_Ack				:    in std_logic;	
				----------------------------------------------------
				-- AXI4 Stream Connection to AD936x SPI Interface --
				----------------------------------------------------
				SPI_Rx_Valid			:  in std_logic;									
				SPI_Rx_Data				:  in std_logic_vector(31 downto 0);		
				SPI_Rx_Ack				: out std_logic;									
				SPI_Tx_Valid			: out std_logic;									
				SPI_Tx_Data				: out std_logic_vector(31 downto 0);		
				SPI_Tx_Ack				:  in std_logic;									
				-----------------------------------------------------
				-- AXI4 Stream Connection to AD936x Data Interface --
				-----------------------------------------------------
				Samples_FIFO_Enable	: out std_logic;									
				Samples_Rx_Valid		:  in std_logic;									
				Samples_Rx_Data		:  in std_logic_vector(31 downto 0);		
				Samples_Rx_Ack			: out std_logic;									
				Samples_Tx_Valid		:  in std_logic;									
				Samples_Tx_Data		:  in std_logic_vector(31 downto 0);		
				Samples_Tx_Ack			: out std_logic;									
				---------------------
				-- control signals --
				---------------------
				AD936x_Activated		: out std_logic);									-- activate AD936x transceiver 
end Protocol;

architecture Behavioral of Protocol is
-- Commands --
constant CMD_SPI_ACCESS			: std_logic_vector(7 downto 0)	:= x"01";
constant CMD_RESET				: std_logic_vector(7 downto 0)	:= x"02";
constant CMD_RX_SAMPLES			: std_logic_vector(7 downto 0)	:= x"03";
constant CMD_TX_SAMPLES			: std_logic_vector(7 downto 0)	:= x"04";
-- Constants --
constant WATCHDOG_MAX																				: natural	:= 40_000_000;
constant BLOCKCOUNTER_MAX																			: natural	:= 1_000_000;
-- Signals --
type States is (Idle, CheckHeader, CheckCommand, Back, SPI_0, SPI_1, SPI_2, SPI_3, Reset_0, Reset_1, Reset_2,
					 Samples_Rx_0, Samples_Rx_1, Samples_Rx_2, Samples_Rx_3, Samples_Rx_4, Samples_Tx_0, Samples_Tx_1, Samples_Tx_2, Samples_Tx_3, Samples_Tx_4);
signal State																							: States;								
signal ParaNumber, ByteNumber, SR_Packet														: std_logic_vector(31 downto 0);	
signal ByteCounter, BlockCounter																	: unsigned(31 downto 0);			
signal SR_Header																						: std_logic_vector(87 downto 0);	
signal Watchdog																						: natural range 0 to 2*WATCHDOG_MAX;
begin

	-------------------------
	-- protocol processing --
	-------------------------
	
	ProtocolManager: process(Clock, Reset_ALow)
	begin
		if Reset_ALow='0' then	State						<= Idle;
										SR_Header				<= (others => '0');
										SR_Packet				<= (others => '0');
										ParaNumber				<= (others => '0');
										ByteCounter 			<= (others => '0');
										Ctrl_Tx_Valid			<= '0';
										Ctrl_Tx_Data			<= (others => '0');
										SPI_Tx_Valid			<= '0';
										SPI_Tx_Data				<= (others => '0');
										AD936x_Activated		<= '0';
										ByteNumber				<= (others => '0');
										Samples_FIFO_Enable	<= '0';
										Watchdog					<= 0;
										BlockCounter			<= (others => '0');
		
		elsif rising_edge(Clock) then
			-- default
			if Ctrl_Tx_Ack='1' then Ctrl_Tx_Valid 	<= '0'; end if;
			if SPI_Tx_Ack='1' then SPI_Tx_Valid 		<= '0'; end if;
			Samples_FIFO_Enable	<= '0';
			AD936x_Activated		<= '1';
			-- increment watchdog - when maximum - change state to prevent blockade
			if Watchdog<2*WATCHDOG_MAX then Watchdog <= Watchdog + 1; else Watchdog <= 0; State <= Back; end if;	
			-- State Machine
			case State is
			
				---------------------------
				-- check protocol header --
				---------------------------
				
				when Idle			=>	-- check for new data byte --
											if Ctrl_Rx_Valid='1' then
												-- shift byte in register --
												SR_Header <= SR_Header(79 downto 0) & Ctrl_Rx_Data;
												-- new state --
												State <= CheckHeader;												
											end if;
											-- reset Watchdog
											Watchdog <= 0;
									
				when CheckHeader	=> -- check for header frame bytes
											if SR_Header(87 downto 56)=x"AB020401" and SR_Header(7 downto 0)=x"BA" then
												-- header detected - set the value of the further incoming bytes in Big Endian Format
												ParaNumber <= SR_Header(15 downto 8) & SR_Header(23 downto 16) & SR_Header(31 downto 24) & SR_Header(39 downto 32);
												-- new state
												State <= CheckCommand;
											else
												-- no header detected - back to idle
												State <= Idle;
											end if;
											
				when CheckCommand	=> -- check command
											case SR_Header(47 downto 40) is
												when CMD_SPI_ACCESS	=> -- spi access --
																				State <= SPI_0;
												when CMD_RESET			=> -- reset transceiver
																				State <= Reset_0;
												when CMD_RX_SAMPLES	=> -- read rx data sample packet --
																				State <= Samples_Rx_0;
												when CMD_TX_SAMPLES	=> -- read tx data sample packet --
																				State <= Samples_Tx_0;																			
												when others 			=> -- default --
																				State <= Idle;
											end case;										
											-- reset counter
											ByteCounter 	<= (others => '0');	
											BlockCounter	<= (others => '0');
											-- reset Watchdog
											Watchdog <= 0;											

				--------------------------
				-- command - SPI access --
				--------------------------
				
				when SPI_0			=> -- check for new byte
											if Ctrl_Rx_Valid='1' then
												-- shift byte in
												SR_Packet <= SR_Packet(23 downto 0) & Ctrl_Rx_Data;
												-- reset Watchdog
												Watchdog <= 0;
												-- next state after 4 bytes
												if ByteCounter<3 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= SPI_1; end if;
											end if;

				when SPI_1			=> -- send data to SPI interface
											if SPI_Tx_Ack='1' then 
												SPI_Tx_Valid 	<= '1'; 
												SPI_Tx_Data 	<= SR_Packet;
												SR_Packet		<= (others => '0');
												State 			<= SPI_2; 
												-- reset Watchdog
												Watchdog 		<= 0;
											end if;
											
				when SPI_2			=> -- check for feedback data
											if SPI_Rx_Valid='1' then 
												SR_Packet 	<= SPI_Rx_Data; 
												State 		<= SPI_3; 
												-- reset Watchdog
												Watchdog 	<= 0;
											end if;
											
				when SPI_3			=> -- feedback packet
											if Ctrl_Tx_Ack='1' then 
												Ctrl_Tx_Valid <= '1'; 
												case to_integer(ByteCounter) is
													when 0	=>	Ctrl_Tx_Data <= x"AB";								
													when 1	=>	Ctrl_Tx_Data <= x"01";		
													when 2	=>	Ctrl_Tx_Data <= SR_Header(55 downto 48);		
													when 3	=>	Ctrl_Tx_Data <= x"02";					
													when 4	=>	Ctrl_Tx_Data <= x"04";					
													when 5	=>	Ctrl_Tx_Data <= CMD_SPI_ACCESS;								-- command
													when 6	=>	Ctrl_Tx_Data <= x"04";		
													when 7	=>	Ctrl_Tx_Data <= x"00";		
													when 8	=>	Ctrl_Tx_Data <= x"00";		
													when 9	=>	Ctrl_Tx_Data <= x"00";		
													when 10	=>	Ctrl_Tx_Data <= x"BA"; 	
													when 11	=>	Ctrl_Tx_Data <= SR_Packet(31 downto 24);
													when 12	=>	Ctrl_Tx_Data <= SR_Packet(23 downto 16);
													when 13	=>	Ctrl_Tx_Data <= SR_Packet(15 downto  8);
													when 14	=>	Ctrl_Tx_Data <= SR_Packet( 7 downto  0);
													when others =>
												end case;
												-- reset Watchdog
												Watchdog <= 0;
												-- increment counter --
												if ByteCounter<14 then ByteCounter <= ByteCounter + 1; else State <= Back; end if;
											end if;
	
				----------------------------------
				-- command - reset transceiver  --
				----------------------------------

				when Reset_0		=> -- Reset Low for 1,2us
											AD936x_Activated <= '0';
											-- next state
											if ByteCounter<30 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= Reset_1; end if;

				when Reset_1		=> -- wait 1,2us before access Transceiver
											if ByteCounter<30 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= Reset_2; end if;
											
				when Reset_2		=> -- feedback packet
											if Ctrl_Tx_Ack='1' then 
												Ctrl_Tx_Valid <= '1'; 
												case to_integer(ByteCounter) is
													when 0	=>	Ctrl_Tx_Data <= x"AB";								
													when 1	=>	Ctrl_Tx_Data <= x"01";		
													when 2	=>	Ctrl_Tx_Data <= SR_Header(55 downto 48);		
													when 3	=>	Ctrl_Tx_Data <= x"02";					
													when 4	=>	Ctrl_Tx_Data <= x"04";					
													when 5	=>	Ctrl_Tx_Data <= CMD_RESET;								-- command
													when 6	=>	Ctrl_Tx_Data <= x"00";		
													when 7	=>	Ctrl_Tx_Data <= x"00";		
													when 8	=>	Ctrl_Tx_Data <= x"00";		
													when 9	=>	Ctrl_Tx_Data <= x"00";		
													when 10	=>	Ctrl_Tx_Data <= x"BA";													
													when others =>
												end case;
												-- reset Watchdog
												Watchdog <= 0;
												-- increment counter --
												if ByteCounter<10 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= Back; end if;
											end if;
				
				--------------------------------------------------------------
				-- command - read rx data sample packet (x complex samples) --
				--------------------------------------------------------------

				when Samples_Rx_0	=> -- check for new byte
											if Ctrl_Rx_Valid='1' then
												-- shift byte in
												SR_Packet <= SR_Packet(23 downto 0) & Ctrl_Rx_Data;
												-- reset Watchdog
												Watchdog <= 0;
												-- increment counter
												if ByteCounter<1 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= Samples_Rx_1; end if;
											end if;

				when Samples_Rx_1	=> -- set number of samples
											ByteNumber <= x"0000" & SR_Packet(7 downto 0) & SR_Packet(15 downto 8);
											-- next state
											State <= Samples_Rx_2;
											
				when Samples_Rx_2	=> -- feedback header
											if Ctrl_Tx_Ack='1' then 
												Ctrl_Tx_Valid <= '1'; 
												case to_integer(ByteCounter) is
													when 0	=>	Ctrl_Tx_Data <= x"AB";								
													when 1	=>	Ctrl_Tx_Data <= x"01";		
													when 2	=>	Ctrl_Tx_Data <= SR_Header(55 downto 48);		
													when 3	=>	Ctrl_Tx_Data <= x"02";					
													when 4	=>	Ctrl_Tx_Data <= x"04";					
													when 5	=>	Ctrl_Tx_Data <= CMD_RX_SAMPLES;								-- command
													when 6	=>	Ctrl_Tx_Data <= ByteNumber( 7 downto 0);		
													when 7	=>	Ctrl_Tx_Data <= ByteNumber(15 downto 8);		
													when 8	=>	Ctrl_Tx_Data <= x"00";		
													when 9	=>	Ctrl_Tx_Data <= x"00";		
													when 10	=>	Ctrl_Tx_Data <= x"BA";												
													when others =>
												end case;
												-- reset Watchdog
												Watchdog <= 0;
												-- increment counter --
												if ByteCounter<10 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= Samples_Rx_3; end if;
											end if;													

				when Samples_Rx_3	=> -- enable sample fifo
											Samples_FIFO_Enable <= '1';
											-- counter to check whether there are sufficient data or not
                                 if BlockCounter<x"00FFFF" then BlockCounter <= BlockCounter + 1; else BlockCounter <= (others => '0'); State <= Samples_Rx_4; end if;
											-- feedback attachment - data = x complex samples
											if Ctrl_Tx_Ack='1' and Samples_Rx_Valid='1' then 
												Ctrl_Tx_Valid <= '1'; 
												case ByteCounter(1 downto 0) is
													when "00"	=> Ctrl_Tx_Data 	<= Samples_Rx_Data(31 downto 24); 
													when "01"	=> Ctrl_Tx_Data 	<= Samples_Rx_Data(23 downto 16); 
													when "10"	=> Ctrl_Tx_Data 	<= Samples_Rx_Data(15 downto  8); 
													when "11"	=> Ctrl_Tx_Data 	<= Samples_Rx_Data( 7 downto  0); 
													when others	=>
												end case;
												-- reset Watchdog
												Watchdog <= 0;
												-- reset counter
												BlockCounter <= (others => '0');
												-- increment counter
												if ByteCounter<unsigned(ByteNumber)-1 then ByteCounter <= ByteCounter + 1; else State <= Back; end if;
											end if;

				when Samples_Rx_4	=> -- check for fifo space
				                     if Ctrl_Tx_Ack='1' then 
												-- fill packet with zeros
				                        Ctrl_Tx_Valid <= '1';
				                        Ctrl_Tx_Data  <= x"00";
				                        -- reset watchdog
                                    Watchdog <= 0;
                                    -- increment counter
                                    if ByteCounter<unsigned(ByteNumber)-1 then ByteCounter <= ByteCounter + 1; else State <= Back; end if;
											end if;				                        

				-----------------------------------------------------------------------
				-- Befehl - Datenpaket abrufen (Sendepfad, x komplexe Abtastwerte) --
				-----------------------------------------------------------------------

				when Samples_Tx_0	=> -- check for new byte
											if Ctrl_Rx_Valid='1' then
												-- shift byte in
												SR_Packet <= SR_Packet(23 downto 0) & Ctrl_Rx_Data;
												-- reset Watchdog
												Watchdog <= 0;
												-- increment counter
												if ByteCounter<1 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= Samples_Tx_1; end if;
											end if;

				when Samples_Tx_1	=> -- set number of samples
											ByteNumber <= x"0000" & SR_Packet(7 downto 0) & SR_Packet(15 downto 8);
											-- next state
											State <= Samples_Tx_2;
											
				when Samples_Tx_2	=> -- feedback header
											if Ctrl_Tx_Ack='1' then 
												Ctrl_Tx_Valid <= '1'; 
												case to_integer(ByteCounter) is
													when 0	=>	Ctrl_Tx_Data <= x"AB";								
													when 1	=>	Ctrl_Tx_Data <= x"01";		
													when 2	=>	Ctrl_Tx_Data <= SR_Header(55 downto 48);		
													when 3	=>	Ctrl_Tx_Data <= x"02";					
													when 4	=>	Ctrl_Tx_Data <= x"04";					
													when 5	=>	Ctrl_Tx_Data <= CMD_TX_SAMPLES;								-- command
													when 6	=>	Ctrl_Tx_Data <= ByteNumber( 7 downto 0);		
													when 7	=>	Ctrl_Tx_Data <= ByteNumber(15 downto 8);		
													when 8	=>	Ctrl_Tx_Data <= x"00";		
													when 9	=>	Ctrl_Tx_Data <= x"00";		
													when 10	=>	Ctrl_Tx_Data <= x"BA";												
													when others =>
												end case;
												-- reset Watchdog
												Watchdog <= 0;
												-- increment counter --
												if ByteCounter<10 then ByteCounter <= ByteCounter + 1; else ByteCounter <= (others => '0'); State <= Samples_Tx_3; end if;
											end if;													

				when Samples_Tx_3	=> -- enable sample fifo
											Samples_FIFO_Enable <= '1';
											-- counter to check whether there are sufficient data or not
                                 if BlockCounter<x"0000FFFF" then BlockCounter <= BlockCounter + 1; else BlockCounter <= (others => '0'); State <= Samples_Tx_4; end if;
											-- feedback attachment - data = x complex samples
											if Ctrl_Tx_Ack='1' and Samples_Tx_Valid='1' then 
												Ctrl_Tx_Valid <= '1'; 
												case ByteCounter(1 downto 0) is
													when "00"	=> Ctrl_Tx_Data 	<= Samples_Tx_Data(31 downto 24); 
													when "01"	=> Ctrl_Tx_Data 	<= Samples_Tx_Data(23 downto 16); 
													when "10"	=> Ctrl_Tx_Data 	<= Samples_Tx_Data(15 downto  8); 
													when "11"	=> Ctrl_Tx_Data 	<= Samples_Tx_Data( 7 downto  0); 
													when others	=>
												end case;
												-- reset Watchdog
												Watchdog <= 0;
												-- reset counter
												BlockCounter <= (others => '0');
												-- increment counter
												if ByteCounter<unsigned(ByteNumber)-1 then ByteCounter <= ByteCounter + 1; else State <= Back; end if;
											end if;

				when Samples_Tx_4	=> -- check for fifo space
				                     if Ctrl_Tx_Ack='1' then 
												-- fill packet with zeros
				                        Ctrl_Tx_Valid <= '1';
				                        Ctrl_Tx_Data  <= x"00";
				                        -- reset watchdog
                                    Watchdog <= 0;
                                    -- increment counter
                                    if ByteCounter<unsigned(ByteNumber)-1 then ByteCounter <= ByteCounter + 1; else State <= Back; end if;
											end if;				                        
														
				-------------------
				-- back to start --
				-------------------
				
				when Back			=> -- reset header register - back to start
											SR_Header <= (others => '0'); State <= Idle;
				when others =>		
									
			end case;
		end if;
	end process ProtocolManager;
	
	
	-- ack data asynchronously
	Ctrl_Rx_Ack <= '1' when State=Idle or State=SPI_0 or State=Samples_Rx_0 or State=Samples_Tx_0 else '0';
	
	SPI_Rx_Ack <= '1' when State=SPI_2 else '0';

	Samples_Rx_Ack <= '1' when State=Samples_Rx_3 and Ctrl_Tx_Ack='1' and ByteCounter(1 downto 0)="11" else '0';

	Samples_Tx_Ack <= '1' when State=Samples_Tx_3 and Ctrl_Tx_Ack='1' and ByteCounter(1 downto 0)="11" else '0';


end Behavioral;

